﻿using MySql.Data.MySqlClient;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DyuClient.Server.Helper {
    public class MySqlHelper {

        public string connectionString;

        public MySqlHelper(string username, string password, string address, string port, string database) {
            connectionString = $"Server={address};Port={port};Database={database};Uid={username};Pwd={password};";
        }

        public bool TestConnection() {
            try {
                using (var connection = new MySqlConnection(connectionString)) {
                    connection.Open();
                    return connection.State == ConnectionState.Open;
                }
            } catch (Exception) {
                return false;
            }
        }

        public DataTable ExecuteQuery(string query) {
            try {
                using (var connection = new MySqlConnection(connectionString)) {
                    connection.Open();
                    using (var command = new MySqlCommand(query, connection)) {
                        using (var adapter = new MySqlDataAdapter(command)) {
                            var dataTable = new DataTable();
                            adapter.Fill(dataTable);
                            return dataTable;
                        }
                    }
                }
            } catch (Exception ex) {
                // 处理异常或记录日志
                throw new Exception("Error executing query: " + ex.Message);
            }
        }

        public bool ExecuteNonQuery(string query) {
            try {
                using (var connection = new MySqlConnection(connectionString)) {
                    connection.Open();
                    using (var command = new MySqlCommand(query, connection)) {
                        var result = command.ExecuteNonQuery();
                        return result > 0;
                    }
                }
            } catch (Exception ex) {
                // 处理异常或记录日志
                throw new Exception("Error executing non-query: " + ex.Message);
            }
        }
    }
}
