﻿using DyuClient.ViewModel.TabPages;
using HandyControl.Controls;
using HelixToolkit.Wpf;
using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Media3D;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace DyuClient.ViewPages.TabPages {
    /// <summary>
    /// Demo1.xaml 的交互逻辑
    /// </summary>
    public partial class Demo1 {
        public Demo1() {
            InitializeComponent();
            DataContext = new Demo1Model();
            // GridLinesVisual3D grid = new GridLinesVisual3D();
            // helixViewport.Children.Add(grid);
            
            // var cameraController = helixViewport.CameraController;
            // if (cameraController != null)
            // {
            //     cameraController.LeftMouseButton = MouseButton.Left;
            //     cameraController.RightMouseButton = MouseButton.Right;
            // }
            var cameraController = helixViewport.CameraController;
            if (cameraController != null)
            {
                // 更改左键行为为旋转
                cameraController.RotateCursor = Cursors.Arrow;

                // 关闭默认的右键行为

            }
        }

        #region 导入导出撤销重置
        //导入
        private void Button_Click(object sender, RoutedEventArgs e) {
            var openFileDialog = new OpenFileDialog {
                Filter = "3D Model Files|*.obj;*.3ds;*.stl;*.ply;*.dae"
            };
            String ModelPath = string.Empty;
            if (openFileDialog.ShowDialog() == true) {
                ModelPath = openFileDialog.FileName;
            } else { 
                return;
            }

            ModelImporter importer = new ModelImporter();

            Model3DGroup group1 = new Model3DGroup();

            var model1 = new ModelVisual3D();

            var mg1 = importer.Load(ModelPath);
            model1.Content = mg1;

            helixViewport.Children.Add(model1);
        }
        //导出
        private void Button_Click_5(object sender, RoutedEventArgs e) {
            string filePath = "C:\\Users\\tongg\\Desktop\\HandyControl-master\\test.stl";
            var exporter = new StlExporter();

            // 获取 HelixViewport3D 中的所有模型
            var models = helixViewport.Children.OfType<ModelVisual3D>().Select(m => m.Content);

            // 创建一个 Model3DGroup 并添加所有模型
            var modelGroup = new Model3DGroup();
            foreach (var model in models) {
                modelGroup.Children.Add(model);
            }

            // 使用导出器将模型导出到文件
            using (var stream = File.Create(filePath)) {
                exporter.Export(modelGroup, stream);
            }

            Growl.Success("好了");
        }
        //清空
        private void Button_Click_1(object sender, RoutedEventArgs e) {
            helixViewport.Children.Clear();
            var light = new DirectionalLight();
            helixViewport.Children.Add(new ModelVisual3D { Content = light });
            // GridLinesVisual3D grid = new GridLinesVisual3D();
            // helixViewport.Children.Add(grid);
            Growl.Success("空了");

        }
        //退回
        private void Button_Click_3(object sender, RoutedEventArgs e) {
            helixViewport.Children.RemoveAt(helixViewport.Children.Count - 1);
        }
        #endregion

        #region 钻石
        private void Button_Click_4(object sender, RoutedEventArgs e) {
            zuanShi(8);
        }

        private void zuanShi(int bian) {
            var meshBuilder = new MeshBuilder(true, true);

            // 定义八边形顶点
            double radius = 3.2915;         //八边形的半径
            double smallRadius = radius / 2; // 小八边形半径
            double topHeight = 1.67;
            double bottomHeight = 4.43;
            double zOffset = 1.67;
            double smallOffset = 1.25; // 小八边形的高度偏移

            Point3DCollection largeVertices = new Point3DCollection();
            Point3DCollection smallVertices = new Point3DCollection();

            // 大八边形顶点
            for (int i = 0; i < bian; i++) {
                double angle = i * Math.PI / (bian / 2); // 45度一边
                largeVertices.Add(new Point3D(radius * Math.Cos(angle), radius * Math.Sin(angle), topHeight));
            }

            // 小八边形顶点
            for (int i = 0; i < bian; i++) {
                double angle = i * Math.PI / (bian / 2); // 45度一边
                smallVertices.Add(new Point3D(smallRadius * Math.Cos(angle), smallRadius * Math.Sin(angle), topHeight + smallOffset));
            }

            // 底部顶点
            Point3D bottomPoint = new Point3D(0, 0, topHeight - bottomHeight);
            Point3D topPoint = new Point3D(0, 0, topHeight );

            // 添加侧面和底部面
            for (int i = 0; i < bian; i++) {
                int nextIndex = (i + 1) % bian;
                AddFace(meshBuilder, topPoint, largeVertices[i], largeVertices[nextIndex]); // 侧面
                AddFace(meshBuilder, bottomPoint, largeVertices[nextIndex], largeVertices[i]); // 底部面
            }

            // 添加较小八边形的面
            AddPolygon(meshBuilder, smallVertices);

            // 连接大八边形和小八边形的侧面
            for (int i = 0; i < bian; i++) {
                int nextIndex = (i + 1) % bian;
                AddFace(meshBuilder, largeVertices[i], largeVertices[nextIndex], smallVertices[nextIndex]); // 侧面1
                AddFace(meshBuilder, largeVertices[i], smallVertices[nextIndex], smallVertices[i]); // 侧面2
            }

            var mesh = meshBuilder.ToMesh();
            mesh.Freeze();

            // 创建材质
            var material = new DiffuseMaterial(new SolidColorBrush(Colors.Cyan));
            var specularMaterial = new SpecularMaterial(new SolidColorBrush(Colors.White), 50);

            var materialGroup = new MaterialGroup();
            materialGroup.Children.Add(material);
            materialGroup.Children.Add(specularMaterial);

            var geometryModel = new GeometryModel3D(mesh, materialGroup);

            // 创建 ModelVisual3D 并将其添加到视图中
            var modelVisual = new ModelVisual3D { Content = geometryModel };

            // 创建光源
            //var light1 = new DirectionalLight(Colors.White, new Vector3D(1, -1, -1));
            //var light2 = new DirectionalLight(Colors.White, new Vector3D(-1, -1, -1));
            //var light3 = new DirectionalLight(Colors.White, new Vector3D(0, -1, 1));
            //var light4 = new DirectionalLight(Colors.White, new Vector3D(1, 1, 1));
            //var light5 = new DirectionalLight(Colors.White, new Vector3D(-1, 1, 1));
            //var light6 = new DirectionalLight(Colors.White, new Vector3D(0, 1, -1));

            //var lightModel1 = new ModelVisual3D { Content = light1 };
            //var lightModel2 = new ModelVisual3D { Content = light2 };
            //var lightModel3 = new ModelVisual3D { Content = light3 };
            //var lightModel4 = new ModelVisual3D { Content = light4 };
            //var lightModel5 = new ModelVisual3D { Content = light5 };
            //var lightModel6 = new ModelVisual3D { Content = light6 };

            //helixViewport.Children.Clear();
            //helixViewport.Children.Add(lightModel1);
            //helixViewport.Children.Add(lightModel2);
            //helixViewport.Children.Add(lightModel3);
            //helixViewport.Children.Add(lightModel4);
            //helixViewport.Children.Add(lightModel5);
            //helixViewport.Children.Add(lightModel6);
            helixViewport.Children.Add(modelVisual);

            // 设置相机视角
            var camera = new PerspectiveCamera {
                Position = new Point3D(10, 10, 10),
                LookDirection = new Vector3D(-10, -10, -10),
                UpDirection = new Vector3D(0, 0, 1),
                FieldOfView = 45
            };
            helixViewport.Camera = camera;

            // GridLinesVisual3D grid = new GridLinesVisual3D();
            // helixViewport.Children.Add(grid);
        }

        //任意边钻石
        private void Button_Click_6(object sender, RoutedEventArgs e) {
            String ss = zsdm.Text;
            int bian = 8;
            if (int.TryParse(ss, out bian)) {
                zuanShi(bian);
            } else {
                Growl.Error("不对");
            }
        }

        #endregion

        #region 曲线曲面

        private void Button_Click_7(object sender, RoutedEventArgs e) {
            // 绘制曲线
            DrawCurve(helixViewport);

            // 绘制曲面
            DrawSurface(helixViewport);
        }
        private void DrawCurve(HelixViewport3D viewport) {
            // 创建一个点集合，用于存储曲线上的点
            var points = new Point3DCollection();

            // 定义曲线上的点
            for (double t = 0; t < 2 * Math.PI; t += 0.1) {
                // 使用参数方程定义曲线 (例如，螺旋线)
                double x = t * Math.Cos(t);
                double y = t * Math.Sin(t);
                double z = t;

                // 将点添加到点集合中
                points.Add(new Point3D(x, y, z));
            }

            // 创建一个线条生成器
            var builder = new MeshBuilder();
            builder.AddTube(points, 0.05, 10, false);

            // 创建一个网格模型
            var mesh = builder.ToMesh(true);
            var material = new DiffuseMaterial(new System.Windows.Media.SolidColorBrush(System.Windows.Media.Colors.Blue));
            var model = new GeometryModel3D(mesh, material);

            // 将模型添加到视图中
            var modelVisual3D = new ModelVisual3D { Content = model };
            viewport.Children.Add(modelVisual3D);
        }

        private void DrawSurface(HelixViewport3D viewport) {
            // 创建一个网格生成器
            var builder = new MeshBuilder();

            // 定义曲面上的点 (例如，z = sin(x) * cos(y))
            int xSteps = 50;
            int ySteps = 50;
            double xMin = -Math.PI;
            double xMax = Math.PI;
            double yMin = -Math.PI;
            double yMax = Math.PI;

            // 创建一个二维数组存储点
            var positions = new Point3D[xSteps, ySteps];

            for (int xi = 0; xi < xSteps; xi++) {
                double x = xMin + (xMax - xMin) * xi / (xSteps - 1);
                for (int yi = 0; yi < ySteps; yi++) {
                    double y = yMin + (yMax - yMin) * yi / (ySteps - 1);
                    double z = Math.Sin(x) * Math.Cos(y);
                    positions[xi, yi] = new Point3D(x, y, z);
                }
            }

            // 创建三角形网格
            for (int xi = 0; xi < xSteps - 1; xi++) {
                for (int yi = 0; yi < ySteps - 1; yi++) {
                    var p0 = positions[xi, yi];
                    var p1 = positions[xi, yi + 1];
                    var p2 = positions[xi + 1, yi];
                    var p3 = positions[xi + 1, yi + 1];

                    // 添加两个三角形构成一个四边形面
                    builder.AddTriangle(p0, p1, p2);
                    builder.AddTriangle(p1, p3, p2);
                }
            }

            // 创建网格模型
            var mesh = builder.ToMesh(true);
            var material = new DiffuseMaterial(new System.Windows.Media.SolidColorBrush(System.Windows.Media.Colors.Red));

            // 设置背面材质与正面材质相同
            var backMaterial = new DiffuseMaterial(new System.Windows.Media.SolidColorBrush(System.Windows.Media.Colors.Red));

            var model = new GeometryModel3D(mesh, material) {
                BackMaterial = backMaterial // 设置背面材质
            };

            // 将模型添加到视图中
            var modelVisual3D = new ModelVisual3D { Content = model };
            viewport.Children.Add(modelVisual3D);
        }

        #endregion
        
        
        private void AddFace(MeshBuilder meshBuilder, Point3D p0, Point3D p1, Point3D p2) {
            // 添加正面
            meshBuilder.AddTriangle(p0, p1, p2);
            Console.Out.WriteLine(Math.Round(p0.X,5)+","+Math.Round(p0.Y,5)+","+Math.Round(p0.Z,5)+";"+
            Math.Round(p1.X,5)+","+Math.Round(p1.Y,5)+","+Math.Round(p1.Z,5)+";"+
            Math.Round(p2.X,5)+","+Math.Round(p2.Y,5)+","+Math.Round(p2.Z,5)+";");
            // 添加背面
            //meshBuilder.AddTriangle(p0, p2, p1);
        }
        private void AddPolygon(MeshBuilder meshBuilder, Point3DCollection vertices) {
            int n = vertices.Count;
            Point3D center = new Point3D(0, 0, vertices[0].Z);

            for (int i = 0; i < n; i++) {
                int nextIndex = (i + 1) % n;
                AddFace(meshBuilder, center, vertices[i], vertices[nextIndex]);
            }
        }
        private void AddLine(Point3D startPoint, Vector3D direction, Color color, double length) {
            var endPoint = startPoint + direction * length;
            var line = new LinesVisual3D {
                Points = new Point3DCollection(new[] { startPoint, endPoint }),
                Color = color,
                Thickness = 2
            };

            helixViewport.Children.Add(line);
        }
        
        #region 自由绘制 直边平面
        private void Button_Click_2(object sender, RoutedEventArgs e) {
            string strAllPoints = textPoint.Text;
            List<Point3D> dPoint = new List<Point3D>();
            var curvedEdges = new Dictionary<(int, int), double>();
            
            string strPoints = string.Empty;
            string strCurvedEdges = string.Empty;
            
            if (strAllPoints.Contains("|"))
            {
                strPoints = strAllPoints.Split("|").First();
                strCurvedEdges = strAllPoints.Split("|").Last();
            }
            else
            {
                strPoints = strAllPoints;
            }

            if (!string.IsNullOrWhiteSpace(strPoints))
            {
                foreach (string point in strPoints.Split(';')) {
                    if (point.Split(',').Length == 3) {
                        double x, y, z;
                        if (!double.TryParse(point.Split(',')[0], out x)) {
                            Growl.Error(point + "，x坐标不对");
                        }
                        if (!double.TryParse(point.Split(',')[1], out y)) {
                            Growl.Error(point + "，y坐标不对");
                        }
                        if (!double.TryParse(point.Split(',')[2], out z)) {
                            Growl.Error(point + "，z坐标不对");
                        }
                        dPoint.Add(new Point3D(x, y, z));
                    } else {
                        Growl.Error(point + "坐标点信息不足或太多");
                    }
                }
            }

            if (!string.IsNullOrWhiteSpace(strCurvedEdges))
            {
                foreach (string point in strCurvedEdges.Split(';')) {
                    if (point.Split(',').Length == 3) {
                        int p1,p2;
                        double edges;
                        if (!int.TryParse(point.Split(',')[0], out p1)) {
                            Growl.Error(point + "，点不合法");
                        }
                        if (!int.TryParse(point.Split(',')[1], out p2)) {
                            Growl.Error(point + "，点不合法");
                        }
                        if (!double.TryParse(point.Split(',')[2], out edges)) {
                            Growl.Error(point + "，曲率不合法");
                        }
                        curvedEdges.Add((p1,p2),edges);
                    } else {
                        Growl.Error(point + "坐标点信息不足或太多");
                    }
                }                
                
            }

            // 创建MeshBuilder对象
            var builder = new MeshBuilder();

            // 添加包含曲线边的多边形
            
            List<Point3D> pointRes = PolygonHelper.AddPolygonWithCurves(builder, dPoint, curvedEdges);
            
            PolygonHelper.AddEdgesToViewport(helixViewport, pointRes, Colors.Red, 2.0);
            
            // 创建并显示模型
            var mesh = builder.ToMesh(true);
            var material = MaterialHelper.CreateMaterial(Colors.Blue);
            var model = new GeometryModel3D(mesh, material);

            // 添加模型到视图中
            // helixViewport.Children.Clear();
            helixViewport.Children.Add(new ModelVisual3D { Content = model });
        }

        
        #endregion


        private void Button_Click_8(object sender, RoutedEventArgs e) {

            // 创建视图对象
            HelixViewport3D viewport = helixViewport; // 你的视图控件名称

            // 调用绘制圆柱弧面的方法
            PolygonHelper.AddCylindricalSurfaceWithTopBottomEdges(viewport, new Point3D(0, 0, 5), 5, 1, 50, Colors.Cyan, Colors.Red, 2.0);
           
            // AddCylindricalSurface
        }


        private void ButtonBase_OnClick(object sender, RoutedEventArgs e)
        {
            string s = rrichTextBox.Text;

            if (string.IsNullOrWhiteSpace(s))
            {
                Growl.Info("??? 输入不能为空");
                return;
            }

            foreach (var strAllPoints in s.Split("\r\n"))
            {
                     // string  = textPoint.Text;
                List<Point3D> dPoint = new List<Point3D>();
                var curvedEdges = new Dictionary<(int, int), double>();
                
                string strPoints = string.Empty;
                string strCurvedEdges = string.Empty;
                
                if (strAllPoints.Contains("|"))
                {
                    strPoints = strAllPoints.Split("|").First();
                    strCurvedEdges = strAllPoints.Split("|").Last();
                }
                else
                {
                    strPoints = strAllPoints;
                }

                if (!string.IsNullOrWhiteSpace(strPoints))
                {
                    foreach (string point in strPoints.Split(';')) {
                        if (point.Split(',').Length == 3) {
                            double x, y, z;
                            if (!double.TryParse(point.Split(',')[0], out x)) {
                                Growl.Error(point + "，x坐标不对");
                            }
                            if (!double.TryParse(point.Split(',')[1], out y)) {
                                Growl.Error(point + "，y坐标不对");
                            }
                            if (!double.TryParse(point.Split(',')[2], out z)) {
                                Growl.Error(point + "，z坐标不对");
                            }
                            dPoint.Add(new Point3D(x, y, z));
                        } else {
                            Growl.Error(point + "坐标点信息不足或太多");
                        }
                    }
                }

                if (!string.IsNullOrWhiteSpace(strCurvedEdges))
                {
                    foreach (string point in strCurvedEdges.Split(';')) {
                        if (point.Split(',').Length == 3) {
                            int p1,p2;
                            double edges;
                            if (!int.TryParse(point.Split(',')[0], out p1)) {
                                Growl.Error(point + "，点不合法");
                            }
                            if (!int.TryParse(point.Split(',')[1], out p2)) {
                                Growl.Error(point + "，点不合法");
                            }
                            if (!double.TryParse(point.Split(',')[2], out edges)) {
                                Growl.Error(point + "，曲率不合法");
                            }
                            curvedEdges.Add((p1,p2),edges);
                        } else {
                            Growl.Error(point + "坐标点信息不足或太多");
                        }
                    }                
                    
                }

                // 创建MeshBuilder对象
                var builder = new MeshBuilder();

                // 添加包含曲线边的多边形
                
                List<Point3D> pointRes = PolygonHelper.AddPolygonWithCurves(builder, dPoint, curvedEdges);
                
                PolygonHelper.AddEdgesToViewport(helixViewport, pointRes, Colors.Red, 2.0);
                
                // 创建并显示模型
                var mesh = builder.ToMesh(true);
                var material = MaterialHelper.CreateMaterial(Colors.Blue);
                var model = new GeometryModel3D(mesh, material);

                // 添加模型到视图中
                // helixViewport.Children.Clear();
                helixViewport.Children.Add(new ModelVisual3D { Content = model });
            }

        }

        private void ButtonBase_OnClick1(object sender, RoutedEventArgs e)
        {
            PolygonHelper.AddBoundingBox(helixViewport, Colors.Chartreuse, 2.0);
        }
        private void ButtonBase_OnClick2(object sender, RoutedEventArgs e)
        {
            PolygonHelper.AddRulerToBoundingBox(helixViewport);
        }
    }
    
    


public class PolygonHelper
{
    
    private static List<List<Point3D>> polygon = new List<List<Point3D>>();
    /// <summary>
    /// 添加包含曲线边的多边形，并绘制正反面。
    /// </summary>
    /// <param name="builder">MeshBuilder对象，用于构建网格。</param>
    /// <param name="points">多边形的顶点列表。</param>
    /// <param name="curvedEdges">指定曲线边的索引和曲率对列表。</param>
    public static List<Point3D> AddPolygonWithCurves(MeshBuilder builder, List<Point3D> points, Dictionary<(int, int), double> curvedEdges)
    {
        if (points == null || points.Count < 3)
        {
            throw new ArgumentException("至少需要3个点来创建一个多边形。");
        }

        // 确保点按正确的顺序排列，避免折叠现象
        EnsureCorrectOrder(points);

        // 创建一个新的点列表来包含曲线生成的点
        var allPoints = new List<Point3D>();

        // 遍历点列表并添加曲线边
        for (int i = 0; i < points.Count; i++)
        {
            int nextIndex = (i + 1) % points.Count;

            // 将当前点加入列表
            allPoints.Add(points[i]);

            // 如果当前边需要生成曲线，则生成曲线点并添加
            if (curvedEdges.ContainsKey((i, nextIndex)))
            {
                double curvature = curvedEdges[(i, nextIndex)];
                var curvePoints = GenerateCurvePoints(points[i], points[nextIndex], curvature, 20);
                allPoints.AddRange(curvePoints);
            }
        }

        // 确保曲线点列表的顺序也正确
        EnsureCorrectOrder(allPoints);
        //allPoints.ForEach((a) => { Console.Out.WriteLine(a.X + "," + a.Y);  });
        // 第一步：绘制包含曲边的曲面
        AddCurvedFace(builder, allPoints);

        return allPoints;
    }

    /// <summary>
    /// 添加曲面到MeshBuilder。
    /// </summary>
    private static void AddCurvedFace(MeshBuilder builder, List<Point3D> points)
    {
        if (points.Count < 3)
        {
            return;
        }

        Point3D start = points[0];
        Point3D? point = null;
        // 手动将多边形拆分为多个三角形进行添加，避免调用AddTriangleFan引发错误
        for (int i = 1; i < points.Count - 1; i++)
        {
            Point3D lastPoint = points[i-1];
            Point3D nowPoint = points[i];
            Point3D nextPoint = points[i+1];

         
            
            List<Point3D> curvedFaces = new List<Point3D>();

            if (polygon.Any(a =>
                {
                    curvedFaces = a;
                    return a.Contains(lastPoint);
                }))
            {
                List<Point3D> topLinePoints = GetPointsOnLineB(curvedFaces);
                int index = curvedFaces.IndexOf(lastPoint);
                builder.AddTriangle(topLinePoints[index], lastPoint, nowPoint);
                builder.AddTriangle(nowPoint, lastPoint, topLinePoints[index]);
                if (index + 1 < topLinePoints.Count && index != -1 && index != 0)
                {
                    builder.AddTriangle(topLinePoints[index], nowPoint, topLinePoints[index+1]);
                    builder.AddTriangle(topLinePoints[index+1], nowPoint, topLinePoints[index]);
                }
                else if (index + 1 < topLinePoints.Count && index != -1 && index == 0)
                {
                    builder.AddTriangle(topLinePoints.First(),nowPoint , lastPoint);
                    builder.AddTriangle(lastPoint,nowPoint , topLinePoints.First());
                    builder.AddTriangle(topLinePoints.First(),curvedFaces[index+1] , topLinePoints[1]);
                    builder.AddTriangle(topLinePoints[1],curvedFaces[index+1] , topLinePoints.First());
                }
                
                if (!polygon.Any(a => a.Contains(nextPoint)) && index != -1)
                {
                    if (point == null)
                    {
                        Console.Out.WriteLine("-----");
                        builder.AddTriangle(topLinePoints.Last(), nextPoint , topLinePoints.First());
                        builder.AddTriangle( topLinePoints.First(), nextPoint ,topLinePoints.Last());
                        point = nextPoint;
                    }
                    else
                    {
                        builder.AddTriangle( (Point3D)point, nextPoint , topLinePoints.First());
                        builder.AddTriangle( topLinePoints.First(), nextPoint ,(Point3D)point);
                        point = nextPoint;
                    }

                }
            }
            else
            {
                builder.AddTriangle(start, nowPoint, nextPoint);
                builder.AddTriangle(nextPoint, nowPoint, start);
            }
        }
    }

    /// <summary>
    /// 生成曲线边的点。
    /// </summary>
    private static List<Point3D> GenerateCurvePoints(Point3D start, Point3D end, double curvature, int segments)
    {
        var controlPoint = GetControlPoint(start, end, curvature);
        var points = new List<Point3D>();

        for (int i = 1; i < segments; i++)
        {
            double t = i / (double)segments;
            Point3D point = GetBezierPoint(start, controlPoint, end, t);
            points.Add(point);
        }

        if (curvature < 0)
        {
            var temp  = new List<Point3D>();
            temp.Add(start);
            temp.AddRange(points);
            temp.Add(end);
            polygon.Add(temp);
        }


        return points;
    }

    /// <summary>
    /// 获取贝塞尔曲线的控制点。
    /// </summary>
    private static Point3D GetControlPoint(Point3D start, Point3D end, double curvature)
    {
        var midPoint = new Point3D((start.X + end.X) / 2, (start.Y + end.Y) / 2, (start.Z + end.Z) / 2);
        var direction = new Vector3D(end.Y - start.Y, start.X - end.X, 0);
        direction.Normalize();

        if (curvature < 0)
        {
            var inwardDirection = new Vector3D(start.Y - end.Y, end.X - start.X, 0);
            inwardDirection.Normalize();
            return midPoint + inwardDirection * Math.Abs(curvature);
        }

        return midPoint + direction * Math.Abs(curvature);
    }

    /// <summary>
    /// 获取贝塞尔曲线上的点。
    /// </summary>
    private static Point3D GetBezierPoint(Point3D start, Point3D control, Point3D end, double t)
    {
        double x = Math.Pow(1 - t, 2) * start.X + 2 * (1 - t) * t * control.X + Math.Pow(t, 2) * end.X;
        double y = Math.Pow(1 - t, 2) * start.Y + 2 * (1 - t) * t * control.Y + Math.Pow(t, 2) * end.Y;
        double z = Math.Pow(1 - t, 2) * start.Z + 2 * (1 - t) * t * control.Z + Math.Pow(t, 2) * end.Z;

        return new Point3D(x, y, z);
    }

    /// <summary>
    /// 确保点的顺序正确，以防止折叠现象。
    /// </summary>
    public static void EnsureCorrectOrder(List<Point3D> points)
    {
        if (points.Count < 3) return;

        double sum = 0;
        for (int i = 0; i < points.Count; i++)
        {
            Point3D current = points[i];
            Point3D next = points[(i + 1) % points.Count];
            sum += (next.X - current.X) * (next.Y + current.Y);
        }

        if (sum > 0)
        {
            points.Reverse();
        }
    }
    
    /// <summary>
    /// 获取 List<Point3D> 中的所有点对应在直线 B 上的点。
    /// </summary>
    /// <param name="points">多边形的顶点列表。</param>
    /// <returns>List<Point3D> 中的所有点在直线 B 上的对应点。</returns>
    public static List<Point3D> GetPointsOnLineB(List<Point3D> points)
    {
        if (points == null || points.Count < 2)
        {
            throw new ArgumentException("至少需要2个点来确定直线。");
        }

        Point3D startPoint = points[0];
        Point3D endPoint = points[points.Count - 1];

        // 计算直线A的方向向量
        Vector3D directionA = new Vector3D(endPoint.X - startPoint.X, endPoint.Y - startPoint.Y, endPoint.Z - startPoint.Z);
        directionA.Normalize();

        // 找到中间点中垂直于直线A的最高点
        double maxDistance = double.MinValue;
        Point3D highestPoint = points[1];
        for (int i = 1; i < points.Count - 1; i++)
        {
            var point = points[i];
            double distance = Vector3D.CrossProduct(new Vector3D(point.X - startPoint.X, point.Y - startPoint.Y, point.Z - startPoint.Z), directionA).Length;
            if (distance > maxDistance)
            {
                maxDistance = distance;
                highestPoint = point;
            }
        }

        // 计算平行于直线A且经过最高点的直线B上的点
        var pointsOnLineB = new List<Point3D>();
        foreach (var point in points)
        {
            double t = ((point.X - highestPoint.X) * directionA.X + (point.Y - highestPoint.Y) * directionA.Y + (point.Z - highestPoint.Z) * directionA.Z) /
                       (Math.Pow(directionA.X, 2) + Math.Pow(directionA.Y, 2) + Math.Pow(directionA.Z, 2));
            double x = highestPoint.X + t * directionA.X;
            double y = highestPoint.Y + t * directionA.Y;
            double z = highestPoint.Z + t * directionA.Z;
            pointsOnLineB.Add(new Point3D(x, y, z));
        }

        return pointsOnLineB;
    }
    
    /// <summary>
    /// 为给定的多边形顶点生成边，并指定边的颜色。
    /// </summary>
    /// <param name="viewport">HelixViewport3D 对象，用于显示 3D 模型。</param>
    /// <param name="points">多边形的顶点列表。</param>
    /// <param name="color">边的颜色。</param>
    /// <param name="thickness">线条的粗细。</param>
    public static void AddEdgesToViewport(HelixViewport3D viewport, List<Point3D> points, Color color, double thickness = 1.0)
    {
        if (points == null || points.Count < 2)
        {
            throw new ArgumentException("至少需要2个点来生成边。");
        }

        for (int i = 0; i < points.Count; i++)
        {
            Point3D startPoint = points[i];
            Point3D endPoint = points[(i + 1) % points.Count]; // 循环形成封闭边

            // 使用 LinesVisual3D 生成边缘线条
            var line = new LinesVisual3D
            {
                Points = new Point3DCollection(new[] { startPoint, endPoint }),
                Color = color,
                Thickness = thickness
            };

            viewport.Children.Add(line);
        }
    }
     /// <summary>
    /// 绘制一个没有顶面和底面的圆柱弧面，并绘制内外两侧及顶底边框。
    /// </summary>
    /// <param name="viewport">HelixViewport3D 对象，用于显示 3D 模型。</param>
    /// <param name="center">圆心坐标。</param>
    /// <param name="radius">圆柱的半径。</param>
    /// <param name="height">圆柱的高度。</param>
    /// <param name="segments">圆周的分段数，越多越光滑。</param>
    /// <param name="surfaceColor">弧面的颜色。</param>
    /// <param name="edgeColor">边框的颜色。</param>
    /// <param name="edgeThickness">边框的粗细。</param>
    public static void AddCylindricalSurfaceWithTopBottomEdges(
        HelixViewport3D viewport, 
        Point3D center, 
        double radius, 
        double height, 
        int segments, 
        Color surfaceColor, 
        Color edgeColor,
        double edgeThickness = 1.0)
    {
        // 创建一个 MeshBuilder 对象来构建网格
        var builder = new MeshBuilder(false, false);

        // 计算圆柱体弧面的顶点
        for (int i = 0; i <= segments; i++)
        {
            double angle = (i * 2 * Math.PI) / segments; // 当前分段的角度
            double x = center.X + radius * Math.Cos(angle);
            double y = center.Y + radius * Math.Sin(angle);

            // 顶点位置：底部和顶部
            var bottomPoint = new Point3D(x, y, center.Z);
            var topPoint = new Point3D(x, y, center.Z + height);

            // 计算下一个分段的点
            int nextIndex = (i + 1) % (segments + 1);
            double nextAngle = (nextIndex * 2 * Math.PI) / segments;
            double nextX = center.X + radius * Math.Cos(nextAngle);
            double nextY = center.Y + radius * Math.Sin(nextAngle);

            var nextBottomPoint = new Point3D(nextX, nextY, center.Z);
            var nextTopPoint = new Point3D(nextX, nextY, center.Z + height);

            if (i < segments)
            {
                // 添加两个三角形，形成一个长方形的面（外面）
                builder.AddQuad(bottomPoint, nextBottomPoint, nextTopPoint, topPoint);

                // 添加两个三角形，形成一个长方形的面（内面反向）
                builder.AddQuad(nextBottomPoint, bottomPoint, topPoint, nextTopPoint);
            }
        }

        // 生成网格
        var mesh = builder.ToMesh();

        // 设置材质
        var material = new DiffuseMaterial(new SolidColorBrush(surfaceColor));

        // 创建 GeometryModel3D 对象来显示弧面
        var geometryModel = new GeometryModel3D(mesh, material);

        // 创建 ModelVisual3D 并将其添加到视图中
        var modelVisual = new ModelVisual3D { Content = geometryModel };
        viewport.Children.Add(modelVisual);

        // 添加顶面和底面的边框
        AddTopBottomEdgesToViewport(viewport, center, radius, height, segments, edgeColor, edgeThickness);
    }

    /// <summary>
    /// 为给定的圆柱弧面生成顶面和底面的边框，并指定边框的颜色和粗细。
    /// </summary>
    /// <param name="viewport">HelixViewport3D 对象，用于显示 3D 模型。</param>
    /// <param name="center">圆心坐标。</param>
    /// <param name="radius">圆柱的半径。</param>
    /// <param name="height">圆柱的高度。</param>
    /// <param name="segments">圆周的分段数。</param>
    /// <param name="color">边框的颜色。</param>
    /// <param name="thickness">边框的粗细。</param>
    private static void AddTopBottomEdgesToViewport(
        HelixViewport3D viewport, 
        Point3D center, 
        double radius, 
        double height, 
        int segments, 
        Color color, 
        double thickness = 1.0)
    {
        // 添加顶面和底面的边框
        var topEdgePoints = new Point3DCollection();
        var bottomEdgePoints = new Point3DCollection();

        for (int i = 0; i < segments; i++)
        {
            double angle = (i * 2 * Math.PI) / segments; // 当前分段的角度
            double x = center.X + radius * Math.Cos(angle);
            double y = center.Y + radius * Math.Sin(angle);

            // 顶部和底部的边框点
            var bottomPoint = new Point3D(x, y, center.Z);
            var topPoint = new Point3D(x, y, center.Z + height);

            bottomEdgePoints.Add(bottomPoint);
            topEdgePoints.Add(topPoint);
        }

        // 添加第一个点到最后，以确保边框闭合
        bottomEdgePoints.Add(bottomEdgePoints[0]);
        topEdgePoints.Add(topEdgePoints[0]);

        // 使用 LinesVisual3D 生成顶面和底面边缘线条，确保边框线条为连续的闭合环
        AddContinuousEdgeLine(viewport, bottomEdgePoints, color, thickness);
        bottomEdgePoints.RemoveAt(0);
        AddContinuousEdgeLine(viewport, bottomEdgePoints, color, thickness);
        AddContinuousEdgeLine(viewport, topEdgePoints, color, thickness);
        topEdgePoints.RemoveAt(0);
        AddContinuousEdgeLine(viewport, topEdgePoints, color, thickness);
    }

    /// <summary>
    /// 为指定的点列表生成连续封闭的边框线。
    /// </summary>
    /// <param name="viewport">HelixViewport3D 对象。</param>
    /// <param name="points">边框的点列表。</param>
    /// <param name="color">边框的颜色。</param>
    /// <param name="thickness">边框的粗细。</param>
    private static void AddContinuousEdgeLine(HelixViewport3D viewport, Point3DCollection points, Color color, double thickness)
    {
        var edgeLine = new LinesVisual3D
        {
            Points = new Point3DCollection(points),
            Color = color,
            Thickness = thickness
        };

        viewport.Children.Add(edgeLine);
    }
     /// <summary>
    /// 根据视图中的模型绘制一个立方体边框，包裹住现有模型。
    /// </summary>
    /// <param name="viewport">HelixViewport3D 对象，用于显示 3D 模型。</param>
    /// <param name="borderColor">边框颜色。</param>
    /// <param name="borderThickness">边框粗细。</param>
    public static void AddBoundingBox(HelixViewport3D viewport, Color borderColor, double borderThickness = 1.0)
    {
        if (viewport.Children.Count == 0)
        {
            Growl.Info("视图中无模型");
            return;
        }

        Rect3D boundingBox = Rect3D.Empty;

        // 计算所有模型的边界框
        foreach (var child in viewport.Children)
        {
            if (child is ModelVisual3D modelVisual && modelVisual.Content != null)
            {
                boundingBox.Union(modelVisual.Content.Bounds);
            }
        }

        if (boundingBox.IsEmpty)
        {
            Growl.Info("视图中无有效模型");
            return;
        }

        // 获取边界框的各个顶点
        Point3D p0 = new Point3D(boundingBox.X, boundingBox.Y, boundingBox.Z);
        Point3D p1 = new Point3D(boundingBox.X + boundingBox.SizeX, boundingBox.Y, boundingBox.Z);
        Point3D p2 = new Point3D(boundingBox.X + boundingBox.SizeX, boundingBox.Y + boundingBox.SizeY, boundingBox.Z);
        Point3D p3 = new Point3D(boundingBox.X, boundingBox.Y + boundingBox.SizeY, boundingBox.Z);
        Point3D p4 = new Point3D(boundingBox.X, boundingBox.Y, boundingBox.Z + boundingBox.SizeZ);
        Point3D p5 = new Point3D(boundingBox.X + boundingBox.SizeX, boundingBox.Y, boundingBox.Z + boundingBox.SizeZ);
        Point3D p6 = new Point3D(boundingBox.X + boundingBox.SizeX, boundingBox.Y + boundingBox.SizeY, boundingBox.Z + boundingBox.SizeZ);
        Point3D p7 = new Point3D(boundingBox.X, boundingBox.Y + boundingBox.SizeY, boundingBox.Z + boundingBox.SizeZ);

        // 使用 LinesVisual3D 绘制立方体边框
        var edgePoints = new Point3DCollection
        {
            p0, p1, p1, p2, p2, p3, p3, p0, // 底面边框
            p4, p5, p5, p6, p6, p7, p7, p4, // 顶面边框
            p0, p4, p1, p5, p2, p6, p3, p7  // 连接顶面和底面的边框
        };

        var boundingBoxEdges = new LinesVisual3D
        {
            Points = edgePoints,
            Color = borderColor,
            Thickness = borderThickness
        };

        viewport.Children.Add(boundingBoxEdges);
    }
     
      /// <summary>
    /// 为给定的立方体边框添加标尺功能。
    /// </summary>
    /// <param name="viewport">HelixViewport3D 对象。</param>
    public static void AddRulerToBoundingBox(HelixViewport3D viewport)
    {
        if (viewport.Children.Count == 0)
        {
            Growl.Info("视图中无模型");
            return;
        }

        Rect3D boundingBox = Rect3D.Empty;

        // 计算所有模型的边界框
        foreach (var child in viewport.Children)
        {
            if (child is ModelVisual3D modelVisual && modelVisual.Content != null)
            {
                boundingBox.Union(modelVisual.Content.Bounds);
            }
        }

        if (boundingBox.IsEmpty)
        {
            Growl.Info("视图中无有效模型");
            return;
        }

        // 获取边界框的各个边长
        double lengthX = boundingBox.SizeX;
        double lengthY = boundingBox.SizeY;
        double lengthZ = boundingBox.SizeZ;

        // 创建文字标签来标记每个边的长度
        AddTextLabelToViewport(viewport, new Point3D(boundingBox.X + lengthX / 2, boundingBox.Y, boundingBox.Z), lengthX.ToString("F2"), Colors.Red);
        AddTextLabelToViewport(viewport, new Point3D(boundingBox.X, boundingBox.Y + lengthY / 2, boundingBox.Z), lengthY.ToString("F2"), Colors.Green);
        AddTextLabelToViewport(viewport, new Point3D(boundingBox.X, boundingBox.Y, boundingBox.Z + lengthZ / 2), lengthZ.ToString("F2"), Colors.Blue);
    }

    /// <summary>
    /// 在视图中添加文字标签。
    /// </summary>
    /// <param name="viewport">HelixViewport3D 对象。</param>
    /// <param name="position">标签位置。</param>
    /// <param name="text">标签文本。</param>
    /// <param name="color">文本颜色。</param>
    private static void AddTextLabelToViewport(HelixViewport3D viewport, Point3D position, string text, Color color)
    {
        var textVisual3D = new BillboardTextVisual3D
        {
            Material = new EmissiveMaterial(new SolidColorBrush(color)),
            Position = position,
            Text = text,
            Foreground = new SolidColorBrush(color),
            FontSize = 20
        };
        viewport.Children.Add(textVisual3D);
    }
}

    


    
}
