﻿using DyuClient.ViewModel.TabPages;
using System.Windows;
using DyuClient.ViewPages.Main;
using HandyControl.Controls;
using MessageBox = System.Windows.MessageBox;

namespace DyuClient.ViewPages.TabPages
{
    /// <summary>
    /// Demo1.xaml 的交互逻辑
    /// </summary>
    public partial class Demo2
    {
        public Demo2()
        {
            InitializeComponent();
            DataContext = new Demo2Model();
        }
        public static MainWindowContent CurrentMainWindowContent { get; set; }
        
        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            LoadResources("CN");
            // 假设已经设置了CurrentMainWindowContent属性，这里触发它重新加载菜单文字
            if (CurrentMainWindowContent!= null)
            {
                CurrentMainWindowContent.ReloadMenuText();
            }
        }

        private void Button_Click_2(object sender, RoutedEventArgs e)
        {
            LoadResources("US");
            // 假设已经设置了CurrentMainWindowContent属性，这里触发它重新加载菜单文字
            if (CurrentMainWindowContent!= null)
            {
                CurrentMainWindowContent.ReloadMenuText();
            }
        }

        private void LoadResources(string resourceKey)
        {
            // 获取当前应用程序的App实例
            var app = (App)Application.Current;

            // 根据传入的resourceKey获取对应的资源字典
            var resourceDictionary = app.Resources[resourceKey] as ResourceDictionary;

            if (resourceDictionary!= null)
            {
                // 获取资源字典的源文件路径（即资源文件的位置）
                string requestedCulture = resourceDictionary.Source.OriginalString;

                // 在已合并的资源字典中查找对应的资源字典
                var mergedDictionary = Application.Current.Resources.MergedDictionaries.FirstOrDefault(d => d.Source.OriginalString.Equals(requestedCulture));

                if (mergedDictionary!= null)
                {
                    // 先移除当前对应的资源字典
                    Application.Current.Resources.MergedDictionaries.Remove(mergedDictionary);

                    // 再添加回去，以更新资源
                    Application.Current.Resources.MergedDictionaries.Add(mergedDictionary);
                }
                else
                {
                    // 如果未找到对应的资源字典，给出提示信息
                    MessageBox.Show($"未找到与 {resourceKey} 对应的资源字典，请检查资源配置。");
                }
            }
            else
            {
                // 如果未找到对应的资源字典（根据传入的resourceKey），给出提示信息
                MessageBox.Show($"未找到对应 {resourceKey} 的资源字典，请检查App.xaml中的资源配置。");
            }

            // 更新UI元素以显示新的资源
            UpdateUIElementsWithNewResources();
        }

        /// <summary>
        /// 根据新加载的资源更新UI元素的文本内容
        /// </summary>
        private void UpdateUIElementsWithNewResources()
        {
            // 获取资源字典中的值并更新各个UI元素的文本内容

            // 示例：假设界面上有一个名为NoDataLabel的Label控件，更新其文本内容
            var noDataText = Application.Current.TryFindResource("NoData") as string;
            if (noDataText!= null)
            {
                NoDataLabel.Content = noDataText;
            }

            // 同理，更新其他需要根据资源更新文本内容的控件
            // 比如，假设还有名为NameLabel、PasswordLabel等的Label控件，以及名为TestConnectionButton的Button控件

            var nameText = Application.Current.TryFindResource("Name") as string;
            if (nameText!= null)
            {
                NameLabel.Content = nameText;
            }

            var passwordText = Application.Current.TryFindResource("Password") as string;
            if (passwordText!= null)
            {
                PasswordLabel.Content = passwordText;
            }

            var portText = Application.Current.TryFindResource("Port") as string;
            if (portText!= null)
            {
                PortLabel.Content = portText;
            }

            var databaseText = Application.Current.TryFindResource("Database") as string;
            if (databaseText!= null)
            {
                DatabaseLabel.Content = databaseText;
            }

            var testConnectionText = Application.Current.TryFindResource("TestConnection") as string;
            if (testConnectionText!= null)
            {
                TestConnectionButton.Content = testConnectionText;
            }
        }

        private void Button_Click_3(object sender, RoutedEventArgs e)
        {
            // 获取资源字典中的值
            var message = Application.Current.TryFindResource("Msg") as string?? "Default Message";

            // 弹出消息提示框，显示动态内容
            MessageBox.Show(message);

            // 使用 Growl 显示动态内容
            Growl.Success(message);
        }
    }
}