﻿using DyuClient.ViewModel.TabPages;
using HandyControl.Controls;
using HandyControl.Expression.Drawing;
using HelixToolkit.Wpf;
using Microsoft.Win32;
using Org.BouncyCastle.Asn1.Ocsp;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Media3D;
using System.Windows.Navigation;
using System.Windows.Shapes;
using static System.Net.Mime.MediaTypeNames;

namespace DyuClient.ViewPages.TabPages
{
    /// <summary>
    /// Demo4.xaml 的交互逻辑
    /// </summary>
    public partial class Demo4
    {
        public Demo4()
        {
            InitializeComponent();
            DataContext = new Demo1Model();
            GridLinesVisual3D grid = new GridLinesVisual3D();
            helixViewport.Children.Add(grid);
            helixViewport.MouseDown += helix_MouseDown;
            ContextMenu cm = new ContextMenu();
            var mi = new MenuItem();
            mi.Header = "显示";
            cm.Items.Add(mi);
            mi.Click += rightClick;
            helixViewport.ContextMenu = cm;

        }
        private void rightClick(object sender, RoutedEventArgs e)
        {
            try
            {
                MenuItem menu = sender as MenuItem;
                if (menu == null)
                    return;

                string name = menu.Header.ToString();
                if (menu.Icon == null || string.IsNullOrEmpty(menu.Icon.ToString()))
                {
                    var childs = this.helixViewport.Children;
                    foreach (var child in childs)
                    {
                        if (child != null && child.GetType() == typeof(Viewport2DVisual3D))
                        {
                            var viewport = (child as Viewport2DVisual3D);
                            if (viewport != null)
                            {
                                Viewport2DVisual3D.SetIsVisualHostMaterial(viewport.Material, true);
                            }

                        }
                    }
                    FormattedText formtxt = new FormattedText("100", CultureInfo.CurrentCulture,
                          FlowDirection.LeftToRight, new Typeface("123"),
                          20, Brushes.Transparent);
                    formtxt.BuildGeometry(new Point(1, 2));
                    menu.Icon = "√";
                }
                else
                {
                    var childs = this.helixViewport.Children;
                    foreach (var child in childs)
                    {
                        if (child != null && child.GetType() == typeof(Viewport2DVisual3D))
                        {
                            var viewport = (child as Viewport2DVisual3D);
                            if (viewport != null)
                            {
                                Viewport2DVisual3D.SetIsVisualHostMaterial(viewport.Material, false);
                            }
                        }
                    }
                    menu.Icon = null;
                }
            }
            catch (Exception ex)
            {
                System.Windows.MessageBox.Show(ex.ToString());
            }

        }


        private void helix_MouseDown(object sender, MouseButtonEventArgs e)
        {
            var visual = this.helixViewport.FindNearestVisual(e.GetPosition(this.helixViewport));


            var model = visual as ModelVisual3D;

            //materialGroup.Children.Add(specularMaterial);

            foreach (var child in this.helixViewport.Children)
            {
                if (child != null && child.GetType() == typeof(ModelVisual3D))
                {
                    var material = new DiffuseMaterial(new SolidColorBrush(Colors.Cyan));
                    //var specularMaterial = new SpecularMaterial(new SolidColorBrush(Colors.Black), 50);

                    var materialGroup = new MaterialGroup();
                    materialGroup.Children.Add(material);
                    GeometryModel3D geo = ((child as ModelVisual3D).Content as GeometryModel3D);
                    if (geo != null)
                    {
                        geo.Material = material;
                    }
                }
            }

            if (model != null)
            {

                GeometryModel3D geo = model.Content as GeometryModel3D;
                var material = new DiffuseMaterial(new SolidColorBrush(Colors.LightYellow));
                //var specularMaterial = new SpecularMaterial(new SolidColorBrush(Colors.Black), 50);

                var materialGroup = new MaterialGroup();
                materialGroup.Children.Add(material);

                geo.Material = materialGroup;
            }
        }
        List<Viewport2DVisual3D> viewport2DVisual3Ds = new List<Viewport2DVisual3D>();
        private Viewport2DVisual3D To3d(Point3DCollection point3Ds, string value)
        {
            Viewport2DVisual3D to3d = new Viewport2DVisual3D();
            DiffuseMaterial diffuseMaterial = new DiffuseMaterial();
            MeshGeometry3D mesh = new MeshGeometry3D();
            mesh.Positions = point3Ds;
            Int32Collection ints = new Int32Collection();
            ints.Add(0);
            ints.Add(2);
            ints.Add(3);
            ints.Add(0);
            ints.Add(1);
            ints.Add(2);
            mesh.TriangleIndices = ints;
            PointCollection points = new PointCollection();
            points.Add(new Point(1, 1));
            points.Add(new Point(1, 0));
            points.Add(new Point(0, 0));
            points.Add(new Point(0, 1));
            mesh.TextureCoordinates = points;
            to3d.Geometry = mesh;
            var text = new TextBlock();
            text.Text = value;
            text.Background = Brushes.Cyan;
            to3d.Visual = text;
            to3d.Material = new DiffuseMaterial(new SolidColorBrush(Colors.Cyan));

            Viewport2DVisual3D.SetIsVisualHostMaterial(to3d.Material, true);
            this.helixViewport.Children.Add(to3d);
            return to3d;
        }
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            var openFileDialog = new OpenFileDialog
            {
                Filter = "3D Model Files|*.obj;*.3ds;*.stl;*.ply;*.dae"
            };
            String ModelPath = string.Empty;
            if (openFileDialog.ShowDialog() == true)
            {
                ModelPath = openFileDialog.FileName;
            }
            else
            {
                return;
            }

            ModelImporter importer = new ModelImporter();

            Model3DGroup group1 = new Model3DGroup();

            var model1 = new ModelVisual3D();

            var mg1 = importer.Load(ModelPath);
            model1.Content = mg1;

            helixViewport.Children.Add(model1);
        }

        private void Button_Click_4(object sender, RoutedEventArgs e)
        {
            zuanShi3(8);

        }

        private void zuanShi(int bian)
        {
            var meshBuilder = new MeshBuilder(true, true);

            // 定义八边形顶点
            double radius = 3.2915;         //八边形的半径
            double smallRadius = radius / 2; // 小八边形半径
            double topHeight = 1.67;
            double bottomHeight = 4.43;
            double zOffset = 1.67;
            double smallOffset = 1.25; // 小八边形的高度偏移

            Point3DCollection largeVertices = new Point3DCollection();
            Point3DCollection smallVertices = new Point3DCollection();

            // 大八边形顶点
            for (int i = 0; i < bian; i++)
            {
                double angle = i * Math.PI / (bian / 2); // 45度一边
                largeVertices.Add(new Point3D(radius * Math.Cos(angle), radius * Math.Sin(angle), topHeight));
            }

            // 小八边形顶点
            for (int i = 0; i < bian; i++)
            {
                double angle = i * Math.PI / (bian / 2); // 45度一边
                smallVertices.Add(new Point3D(smallRadius * Math.Cos(angle), smallRadius * Math.Sin(angle), topHeight + smallOffset));
            }

            // 底部顶点
            Point3D bottomPoint = new Point3D(0, 0, topHeight - bottomHeight);
            Point3D topPoint = new Point3D(0, 0, topHeight);

            // 添加侧面和底部面
            for (int i = 0; i < bian; i++)
            {
                int nextIndex = (i + 1) % bian;
                AddFace(meshBuilder, topPoint, largeVertices[i], largeVertices[nextIndex]); // 侧面
                AddFace(meshBuilder, bottomPoint, largeVertices[nextIndex], largeVertices[i]); // 底部面
            }

            // 添加较小八边形的面
            AddPolygon(meshBuilder, smallVertices);

            // 连接大八边形和小八边形的侧面
            for (int i = 0; i < bian; i++)
            {
                int nextIndex = (i + 1) % bian;
                AddFace(meshBuilder, largeVertices[i], largeVertices[nextIndex], smallVertices[nextIndex]); // 侧面1
                AddFace(meshBuilder, largeVertices[i], smallVertices[nextIndex], smallVertices[i]); // 侧面2
            }

            var mesh = meshBuilder.ToMesh();
            mesh.Freeze();

            // 创建材质
            var material = new DiffuseMaterial(new SolidColorBrush(Colors.Cyan));
            var specularMaterial = new SpecularMaterial(new SolidColorBrush(Colors.White), 50);

            var materialGroup = new MaterialGroup();
            materialGroup.Children.Add(material);
            materialGroup.Children.Add(specularMaterial);

            var geometryModel = new GeometryModel3D(mesh, materialGroup);

            // 创建 ModelVisual3D 并将其添加到视图中
            var modelVisual = new ModelVisual3D { Content = geometryModel };

            //aaa.Geometry = mesh;
            //var text = new TextBlock();
            //text.Text = "300";
            //text.FontSize = 10;
            //aaa.Visual = text;
            //Viewport2DVisual3D.SetIsVisualHostMaterial(aaa.Material, true);
            // 创建光源
            //var light1 = new DirectionalLight(Colors.White, new Vector3D(1, -1, -1));
            //var light2 = new DirectionalLight(Colors.White, new Vector3D(-1, -1, -1));
            //var light3 = new DirectionalLight(Colors.White, new Vector3D(0, -1, 1));
            //var light4 = new DirectionalLight(Colors.White, new Vector3D(1, 1, 1));
            //var light5 = new DirectionalLight(Colors.White, new Vector3D(-1, 1, 1));
            //var light6 = new DirectionalLight(Colors.White, new Vector3D(0, 1, -1));

            //var lightModel1 = new ModelVisual3D { Content = light1 };
            //var lightModel2 = new ModelVisual3D { Content = light2 };
            //var lightModel3 = new ModelVisual3D { Content = light3 };
            //var lightModel4 = new ModelVisual3D { Content = light4 };
            //var lightModel5 = new ModelVisual3D { Content = light5 };
            //var lightModel6 = new ModelVisual3D { Content = light6 };

            //helixViewport.Children.Clear();
            //helixViewport.Children.Add(lightModel1);
            //helixViewport.Children.Add(lightModel2);
            //helixViewport.Children.Add(lightModel3);
            //helixViewport.Children.Add(lightModel4);
            //helixViewport.Children.Add(lightModel5);
            //helixViewport.Children.Add(lightModel6);
            helixViewport.Children.Add(modelVisual);

            // 设置相机视角
            var camera = new PerspectiveCamera
            {
                Position = new Point3D(10, 10, 10),
                LookDirection = new Vector3D(-10, -10, -10),
                UpDirection = new Vector3D(0, 0, 1),
                FieldOfView = 45
            };
            helixViewport.Camera = camera;

            GridLinesVisual3D grid = new GridLinesVisual3D();
            helixViewport.Children.Add(grid);
        }
        private void zuanShi2(int bian)
        {
            //var meshBuilder = new MeshBuilder(true, true);

            //// 定义八边形顶点
            //double radius = 3.2915;         //八边形的半径
            //double smallRadius = radius / 2; // 小八边形半径
            //double topHeight = 1.67;
            //double bottomHeight = 4.43;
            //double zOffset = 1.67;
            //double smallOffset = 1.25; // 小八边形的高度偏移

            //Point3DCollection largeVertices = new Point3DCollection();
            //Point3DCollection smallVertices = new Point3DCollection();

            //// 大八边形顶点
            //for (int i = 0; i < bian; i++)
            //{
            //    double angle = i * Math.PI / (bian / 2); // 45度一边
            //    largeVertices.Add(new Point3D(radius * Math.Cos(angle), radius * Math.Sin(angle), topHeight));
            //}

            //// 小八边形顶点
            //for (int i = 0; i < bian; i++)
            //{
            //    double angle = i * Math.PI / (bian / 2); // 45度一边
            //    smallVertices.Add(new Point3D(smallRadius * Math.Cos(angle), smallRadius * Math.Sin(angle), topHeight + smallOffset));
            //}

            //// 底部顶点
            //Point3D bottomPoint = new Point3D(0, 0, topHeight - bottomHeight);
            //Point3D topPoint = new Point3D(0, 0, topHeight);

            //// 添加侧面和底部面
            //for (int i = 0; i < bian; i++)
            //{
            //    int nextIndex = (i + 1) % bian;
            //    AddFace(meshBuilder, topPoint, largeVertices[i], largeVertices[nextIndex]); // 侧面
            //    AddFace(meshBuilder, bottomPoint, largeVertices[nextIndex], largeVertices[i]); // 底部面
            //}

            //// 添加较小八边形的面
            //AddPolygon(meshBuilder, smallVertices);

            //// 连接大八边形和小八边形的侧面
            //for (int i = 0; i < bian; i++)
            //{
            //    int nextIndex = (i + 1) % bian;
            //    AddFace(meshBuilder, largeVertices[i], largeVertices[nextIndex], smallVertices[nextIndex]); // 侧面1
            //    AddFace(meshBuilder, largeVertices[i], smallVertices[nextIndex], smallVertices[i]); // 侧面2
            //}

            //var mesh = meshBuilder.ToMesh();
            //mesh.Freeze();
            // 创建材质
            var material = new DiffuseMaterial(new SolidColorBrush(Colors.Cyan));
            //var specularMaterial = new SpecularMaterial(new SolidColorBrush(Colors.White), 50);

            var materialGroup = new MaterialGroup();
            materialGroup.Children.Add(material);
            //materialGroup.Children.Add(specularMaterial);
            MeshGeometry3D mesh = new MeshGeometry3D();

            // 定义钻石的顶点
            mesh.Positions.Add(new Point3D(0, 0, 0)); // 顶点A
            mesh.Positions.Add(new Point3D(1, 1, 1)); // 顶点B
            mesh.Positions.Add(new Point3D(-1, 1, 1)); // 顶点C
            mesh.Positions.Add(new Point3D(-1, -1, 1)); // 顶点D
            mesh.Positions.Add(new Point3D(1, -1, 1)); // 顶点E
            mesh.Positions.Add(new Point3D(0, 0, -1)); // 顶点F
            // 定义钻石的面
            mesh.TriangleIndices.Add(0); // 面1
            mesh.TriangleIndices.Add(1);
            mesh.TriangleIndices.Add(2);



            var geometryModel = new GeometryModel3D(mesh, materialGroup);

            // 创建 ModelVisual3D 并将其添加到视图中
            var modelVisual = new ModelVisual3D { Content = geometryModel };
            helixViewport.Children.Add(modelVisual);

            mesh = new MeshGeometry3D();

            // 定义钻石的顶点
            mesh.Positions.Add(new Point3D(0, 0, 0)); // 顶点A
            mesh.Positions.Add(new Point3D(1, 1, 1)); // 顶点B
            mesh.Positions.Add(new Point3D(-1, 1, 1)); // 顶点C
            mesh.Positions.Add(new Point3D(-1, -1, 1)); // 顶点D
            mesh.Positions.Add(new Point3D(1, -1, 1)); // 顶点E
            mesh.Positions.Add(new Point3D(0, 0, -1)); // 顶点F
            mesh.TriangleIndices.Add(0); // 面2
            mesh.TriangleIndices.Add(2);
            mesh.TriangleIndices.Add(3);
            geometryModel = new GeometryModel3D(mesh, materialGroup);

            // 创建 ModelVisual3D 并将其添加到视图中
            modelVisual = new ModelVisual3D { Content = geometryModel };
            helixViewport.Children.Add(modelVisual);

            //mesh.TriangleIndices.Add(0); // 面3
            //mesh.TriangleIndices.Add(3);
            //mesh.TriangleIndices.Add(4);

            //mesh.TriangleIndices.Add(0); // 面4
            //mesh.TriangleIndices.Add(4);
            //mesh.TriangleIndices.Add(1);

            //mesh.TriangleIndices.Add(1); // 面5
            //mesh.TriangleIndices.Add(5);
            //mesh.TriangleIndices.Add(2);

            //mesh.TriangleIndices.Add(2); // 面6
            //mesh.TriangleIndices.Add(5);
            //mesh.TriangleIndices.Add(3);

            //mesh.TriangleIndices.Add(3); // 面7
            //mesh.TriangleIndices.Add(5);
            //mesh.TriangleIndices.Add(4);

            //mesh.TriangleIndices.Add(4); // 面8
            //mesh.TriangleIndices.Add(5);
            //mesh.TriangleIndices.Add(1);



            // 创建光源
            //var light1 = new DirectionalLight(Colors.White, new Vector3D(1, -1, -1));
            //var light2 = new DirectionalLight(Colors.White, new Vector3D(-1, -1, -1));
            //var light3 = new DirectionalLight(Colors.White, new Vector3D(0, -1, 1));
            //var light4 = new DirectionalLight(Colors.White, new Vector3D(1, 1, 1));
            //var light5 = new DirectionalLight(Colors.White, new Vector3D(-1, 1, 1));
            //var light6 = new DirectionalLight(Colors.White, new Vector3D(0, 1, -1));

            //var lightModel1 = new ModelVisual3D { Content = light1 };
            //var lightModel2 = new ModelVisual3D { Content = light2 };
            //var lightModel3 = new ModelVisual3D { Content = light3 };
            //var lightModel4 = new ModelVisual3D { Content = light4 };
            //var lightModel5 = new ModelVisual3D { Content = light5 };
            //var lightModel6 = new ModelVisual3D { Content = light6 };

            //helixViewport.Children.Clear();
            //helixViewport.Children.Add(lightModel1);
            //helixViewport.Children.Add(lightModel2);
            //helixViewport.Children.Add(lightModel3);
            //helixViewport.Children.Add(lightModel4);
            //helixViewport.Children.Add(lightModel5);
            //helixViewport.Children.Add(lightModel6);


            // 设置相机视角
            var camera = new PerspectiveCamera
            {
                Position = new Point3D(10, 10, 10),
                LookDirection = new Vector3D(-10, -10, -10),
                UpDirection = new Vector3D(0, 0, 1),
                FieldOfView = 45
            };
            helixViewport.Camera = camera;

            GridLinesVisual3D grid = new GridLinesVisual3D();
            helixViewport.Children.Add(grid);
        }

        private void zuanShi3(int bian)
        {
            var meshBuilder = new MeshBuilder(true, true);

            // 定义八边形顶点
            double radius = 3.2915;         //八边形的半径
            double smallRadius = radius / 2; // 小八边形半径
            double topHeight = 1.67;
            double bottomHeight = 4.43;
            double zOffset = 1.67;
            double smallOffset = 1.25; // 小八边形的高度偏移

            Point3DCollection largeVertices = new Point3DCollection();
            Point3DCollection smallVertices = new Point3DCollection();

            // 创建材质
            var material = new DiffuseMaterial(new SolidColorBrush(Colors.Cyan));
            //var specularMaterial = new SpecularMaterial(new SolidColorBrush(Colors.White), 50);

            var materialGroup = new MaterialGroup();
            materialGroup.Children.Add(material);

            // 大八边形顶点
            for (int i = 0; i < bian; i++)
            {
                double angle = i * Math.PI / (bian / 2); // 45度一边
                largeVertices.Add(new Point3D(radius * Math.Cos(angle), radius * Math.Sin(angle), topHeight));
            }

            // 小八边形顶点
            for (int i = 0; i < bian; i++)
            {
                double angle = i * Math.PI / (bian / 2); // 45度一边
                smallVertices.Add(new Point3D(smallRadius * Math.Cos(angle), smallRadius * Math.Sin(angle), topHeight + smallOffset));
            }

            // 底部顶点
            Point3D bottomPoint = new Point3D(0, 0, topHeight - bottomHeight);
            Point3D topPoint = new Point3D(0, 0, topHeight);

            meshBuilder = new MeshBuilder(true, true);
            var mesh = meshBuilder.ToMesh();
            mesh.Freeze();
            var geometryModel = new GeometryModel3D(mesh, materialGroup);

            // 创建 ModelVisual3D 并将其添加到视图中
            var modelVisual = new ModelVisual3D { Content = geometryModel };
            // 添加侧面和底部面
            for (int i = 0; i < bian; i++)
            {
                int nextIndex = (i + 1) % bian;
                AddFace(meshBuilder, topPoint, largeVertices[i], largeVertices[nextIndex]); // 侧面
                mesh = meshBuilder.ToMesh();
                mesh.Freeze();
                geometryModel = new GeometryModel3D(mesh, materialGroup);
                // 创建 ModelVisual3D 并将其添加到视图中
                modelVisual = new ModelVisual3D { Content = geometryModel };
                helixViewport.Children.Add(modelVisual);
                meshBuilder = new MeshBuilder(true, true);
                AddFace(meshBuilder, bottomPoint, largeVertices[nextIndex], largeVertices[i]); // 底部面
                mesh = meshBuilder.ToMesh();
                mesh.Freeze();
                geometryModel = new GeometryModel3D(mesh, materialGroup);
                // 创建 ModelVisual3D 并将其添加到视图中
                modelVisual = new ModelVisual3D { Content = geometryModel };
                helixViewport.Children.Add(modelVisual);
                meshBuilder = new MeshBuilder(true, true);

            }

            // 添加较小八边形的面
            AddPolygon(meshBuilder, smallVertices);
            mesh = meshBuilder.ToMesh();
            mesh.Freeze();
            geometryModel = new GeometryModel3D(mesh, materialGroup);
            // 创建 ModelVisual3D 并将其添加到视图中
            modelVisual = new ModelVisual3D { Content = geometryModel };
            helixViewport.Children.Add(modelVisual);
            meshBuilder = new MeshBuilder(true, true);
            // 连接大八边形和小八边形的侧面
            for (int i = 0; i < bian; i++)
            {
                int nextIndex = (i + 1) % bian;
                AddFace(meshBuilder, largeVertices[i], largeVertices[nextIndex], smallVertices[nextIndex]); // 侧面1
                AddFace(meshBuilder, largeVertices[i], smallVertices[nextIndex], smallVertices[i]); // 侧面2
                mesh = meshBuilder.ToMesh();
                mesh.Freeze();
                geometryModel = new GeometryModel3D(mesh, materialGroup);
                // 创建 ModelVisual3D 并将其添加到视图中
                modelVisual = new ModelVisual3D { Content = geometryModel };
                helixViewport.Children.Add(modelVisual);
                meshBuilder = new MeshBuilder(true, true);
            }

            //var mesh = meshBuilder.ToMesh();
            //mesh.Freeze();

            //var geometryModel = new GeometryModel3D(mesh, materialGroup);

            //// 创建 ModelVisual3D 并将其添加到视图中
            //var modelVisual = new ModelVisual3D { Content = geometryModel };

            //helixViewport.Children.Add(modelVisual);

            // 设置相机视角
            var camera = new PerspectiveCamera
            {
                Position = new Point3D(10, 10, 10),
                LookDirection = new Vector3D(-10, -10, -10),
                UpDirection = new Vector3D(0, 0, 1),
                FieldOfView = 45
            };
            helixViewport.Camera = camera;

            GridLinesVisual3D grid = new GridLinesVisual3D();
            helixViewport.Children.Add(grid);
        }
        private void AddFace(MeshBuilder meshBuilder, Point3D p0, Point3D p1, Point3D p2)
        {
            // 添加正面
            meshBuilder.AddTriangle(p0, p1, p2);
            // 添加背面
            meshBuilder.AddTriangle(p0, p2, p1);

        }
        private void AddPolygon(MeshBuilder meshBuilder, Point3DCollection vertices)
        {
            int n = vertices.Count;
            Point3D center = new Point3D(0, 0, vertices[0].Z);

            for (int i = 0; i < n; i++)
            {
                int nextIndex = (i + 1) % n;
                AddFace(meshBuilder, center, vertices[i], vertices[nextIndex]);
            }
        }

        private void Button_Click_5(object sender, RoutedEventArgs e)
        {
            string filePath = "C:\\Users\\tongg\\Desktop\\HandyControl-master\\test.stl";
            var exporter = new StlExporter();

            // 获取 HelixViewport3D 中的所有模型
            var models = helixViewport.Children.OfType<ModelVisual3D>().Select(m => m.Content);

            // 创建一个 Model3DGroup 并添加所有模型
            var modelGroup = new Model3DGroup();
            foreach (var model in models)
            {
                modelGroup.Children.Add(model);
            }

            // 使用导出器将模型导出到文件
            using (var stream = File.Create(filePath))
            {
                exporter.Export(modelGroup, stream);
            }

            Growl.Success("好了");
        }

        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            helixViewport.Children.Clear();
            var light = new DirectionalLight();
            helixViewport.Children.Add(new ModelVisual3D { Content = light });
            GridLinesVisual3D grid = new GridLinesVisual3D();
            helixViewport.Children.Add(grid);
            Growl.Success("空了");

        }

        private void Button_Click_3(object sender, RoutedEventArgs e)
        {
            helixViewport.Children.RemoveAt(helixViewport.Children.Count - 1);
        }

        private void Button_Click_2(object sender, RoutedEventArgs e)
        {

        }

        private void Button_Click_6(object sender, RoutedEventArgs e)
        {
            String ss = zsdm.Text;
            int bian = 8;
            if (int.TryParse(ss, out bian))
            {
                zuanShi(bian);
            }
            else
            {
                Growl.Success("空了");
            }
        }
    }
}
